@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2013 Wandenberg Peixoto <wandenberg@gmail.com>, Rogério Carvalho Schneider <stockrt@gmail.com>
+ * Copyright (C) 2010-2015 Wandenberg Peixoto <wandenberg@gmail.com>, Rogério Carvalho Schneider <stockrt@gmail.com>
  *
  * This file is part of Nginx Push Stream Module.
  *
@@ -46,6 +46,7 @@ typedef enum {
     PUSH_STREAM_TEMPLATE_PART_TYPE_EVENT_TYPE,
     PUSH_STREAM_TEMPLATE_PART_TYPE_CHANNEL,
     PUSH_STREAM_TEMPLATE_PART_TYPE_TEXT,
+    PUSH_STREAM_TEMPLATE_PART_TYPE_SIZE,
     PUSH_STREAM_TEMPLATE_PART_TYPE_LITERAL
 } ngx_http_push_stream_template_part_type;
 
@@ -68,6 +69,7 @@ typedef struct {
     ngx_uint_t                      qtd_event_type;
     ngx_uint_t                      qtd_channel;
     ngx_uint_t                      qtd_text;
+    ngx_uint_t                      qtd_size;
     ngx_uint_t                      qtd_tag;
     ngx_uint_t                      qtd_time;
     size_t                          literal_len;
@@ -76,6 +78,7 @@ typedef struct {
 typedef struct ngx_http_push_stream_msg_s ngx_http_push_stream_msg_t;
 typedef struct ngx_http_push_stream_shm_data_s ngx_http_push_stream_shm_data_t;
 typedef struct ngx_http_push_stream_global_shm_data_s ngx_http_push_stream_global_shm_data_t;
+typedef struct ngx_http_push_stream_channel_s ngx_http_push_stream_channel_t;
 
 typedef struct {
     ngx_flag_t                      enabled;
@@ -92,6 +95,7 @@ typedef struct {
     ngx_uint_t                      max_channel_id_length;
     ngx_queue_t                     msg_templates;
     ngx_flag_t                      timeout_with_body;
+    ngx_str_t                       events_channel_id;
     ngx_regex_t                    *backtrack_parser_regex;
     ngx_http_push_stream_msg_t     *ping_msg;
     ngx_http_push_stream_msg_t     *longpooling_timeout_msg;
@@ -115,6 +119,7 @@ typedef struct {
     ngx_msec_t                      longpolling_connection_ttl;
     ngx_flag_t                      websocket_allow_publish;
     ngx_flag_t                      channel_info_on_publish;
+    ngx_flag_t                      allow_connections_to_events_channel;
     ngx_http_complex_value_t       *last_received_message_time;
     ngx_http_complex_value_t       *last_received_message_tag;
     ngx_http_complex_value_t       *last_event_id;
@@ -156,8 +161,7 @@ typedef struct {
     ngx_uint_t                          subscribers;
 } ngx_http_push_stream_pid_queue_t;
 
-// our typecast-friendly rbtree node (channel)
-typedef struct {
+struct ngx_http_push_stream_channel_s {
     ngx_rbtree_node_t                   node;
     ngx_queue_t                         queue;
     ngx_str_t                           id;
@@ -171,9 +175,10 @@ typedef struct {
     time_t                              expires;
     ngx_flag_t                          deleted;
     ngx_flag_t                          wildcard;
+    char                                for_events;
     ngx_http_push_stream_msg_t         *channel_deleted_message;
     ngx_shmtx_t                        *mutex;
-} ngx_http_push_stream_channel_t;
+};
 
 typedef struct {
     ngx_queue_t                         queue;
@@ -218,8 +223,11 @@ typedef struct {
     uint64_t payload_len;
     u_char  header[8];
     u_char *payload;
-    u_char *last;
     ngx_uint_t step;
+    ngx_buf_t  buf;
+    ngx_str_t consolidated;
+    unsigned char fragmented:1;
+    unsigned char last_fragment:1;
 } ngx_http_push_stream_frame_t;
 
 typedef struct {
@@ -227,6 +235,7 @@ typedef struct {
     ngx_event_t                        *ping_timer;
     ngx_http_push_stream_subscriber_t  *subscriber;
     ngx_flag_t                          longpolling;
+    ngx_flag_t                          message_sent;
     ngx_pool_t                         *temp_pool;
     ngx_chain_t                        *free;
     ngx_chain_t                        *busy;
@@ -279,6 +288,7 @@ struct ngx_http_push_stream_shm_data_s {
     ngx_queue_t                             channels_to_delete;
     ngx_shmtx_t                             channels_to_delete_mutex;
     ngx_shmtx_sh_t                          channels_to_delete_lock;
+    ngx_uint_t                              channels_in_delete; // # of channels in to delete queue
     ngx_uint_t                              channels_in_trash;  // # of channels in trash queue
     ngx_uint_t                              messages_in_trash;  // # of messages in trash queue
     ngx_http_push_stream_worker_data_t      ipc[NGX_MAX_PROCESSES]; // interprocess stuff
@@ -295,6 +305,9 @@ struct ngx_http_push_stream_shm_data_s {
     ngx_shmtx_sh_t                          channels_lock[10];
     ngx_shmtx_t                             cleanup_mutex;
     ngx_shmtx_sh_t                          cleanup_lock;
+    ngx_shmtx_t                             events_channel_mutex;
+    ngx_shmtx_sh_t                          events_channel_lock;
+    ngx_http_push_stream_channel_t         *events_channel;
 };
 
 ngx_shm_zone_t     *ngx_http_push_stream_global_shm_zone = NULL;
@@ -319,6 +332,8 @@ static const ngx_str_t NGX_HTTP_PUSH_STREAM_TOO_MUCH_WILDCARD_CHANNELS = ngx_str
 static const ngx_str_t NGX_HTTP_PUSH_STREAM_TOO_SUBSCRIBERS_PER_CHANNEL = ngx_string("Subscribers limit per channel has been exceeded.");
 static const ngx_str_t NGX_HTTP_PUSH_STREAM_CANNOT_CREATE_CHANNELS = ngx_string("Subscriber could not create channels.");
 static const ngx_str_t NGX_HTTP_PUSH_STREAM_NUMBER_OF_CHANNELS_EXCEEDED_MESSAGE = ngx_string("Number of channels were exceeded.");
+static const ngx_str_t NGX_HTTP_PUSH_STREAM_INTERNAL_ONLY_EVENTS_CHANNEL_MESSAGE = ngx_string("Only internal routines can change events channel.");
+static const ngx_str_t NGX_HTTP_PUSH_STREAM_SUBSCRIPTION_EVENTS_CHANNEL_FORBIDDEN_MESSAGE = ngx_string("Subscription to events channel is not allowed.");
 static const ngx_str_t NGX_HTTP_PUSH_STREAM_NO_MANDATORY_HEADERS_MESSAGE = ngx_string("Don't have at least one of the mandatory headers: Connection, Upgrade, Sec-WebSocket-Key and Sec-WebSocket-Version");
 static const ngx_str_t NGX_HTTP_PUSH_STREAM_WRONG_WEBSOCKET_VERSION_MESSAGE = ngx_string("Version not supported. Supported versions: 8, 13");
 static const ngx_str_t NGX_HTTP_PUSH_STREAM_CHANNEL_DELETED = ngx_string("Channel deleted.");
@@ -392,11 +407,12 @@ static const ngx_str_t  NGX_HTTP_PUSH_STREAM_MODE_WEBSOCKET   = ngx_string("webs
 #define NGX_HTTP_PUSH_STREAM_WEBSOCKET_TEXT_OPCODE  0x1
 #define NGX_HTTP_PUSH_STREAM_WEBSOCKET_CLOSE_OPCODE 0x8
 #define NGX_HTTP_PUSH_STREAM_WEBSOCKET_PING_OPCODE  0x9
-#define NGX_HTTP_PUSH_STREAM_WEBSOCKET_P0NG_OPCODE  0xA
+#define NGX_HTTP_PUSH_STREAM_WEBSOCKET_PONG_OPCODE  0xA
 
 static const u_char NGX_HTTP_PUSH_STREAM_WEBSOCKET_TEXT_LAST_FRAME_BYTE    =  NGX_HTTP_PUSH_STREAM_WEBSOCKET_TEXT_OPCODE  | (NGX_HTTP_PUSH_STREAM_WEBSOCKET_LAST_FRAME << 4);
 static const u_char NGX_HTTP_PUSH_STREAM_WEBSOCKET_CLOSE_LAST_FRAME_BYTE[] = {NGX_HTTP_PUSH_STREAM_WEBSOCKET_CLOSE_OPCODE | (NGX_HTTP_PUSH_STREAM_WEBSOCKET_LAST_FRAME << 4), 0x00};
 static const u_char NGX_HTTP_PUSH_STREAM_WEBSOCKET_PING_LAST_FRAME_BYTE[]  = {NGX_HTTP_PUSH_STREAM_WEBSOCKET_PING_OPCODE  | (NGX_HTTP_PUSH_STREAM_WEBSOCKET_LAST_FRAME << 4), 0x00};
+static const u_char NGX_HTTP_PUSH_STREAM_WEBSOCKET_PONG_LAST_FRAME_BYTE[]  = {NGX_HTTP_PUSH_STREAM_WEBSOCKET_PONG_OPCODE  | (NGX_HTTP_PUSH_STREAM_WEBSOCKET_LAST_FRAME << 4), 0x00};
 static const u_char NGX_HTTP_PUSH_STREAM_WEBSOCKET_PAYLOAD_LEN_16_BYTE   = 126;
 static const u_char NGX_HTTP_PUSH_STREAM_WEBSOCKET_PAYLOAD_LEN_64_BYTE   = 127;
 
